//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include "Main.h"
#include "Progress.h"
#include "TOverwriteDlg.h"

#include "ConvertThread.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------

//   Important: Methods and properties of objects in VCL can only be
//   used in a method called using Synchronize, for example:
//
//      Synchronize(UpdateCaption);
//
//   where UpdateCaption could look like:
//
//      void __fastcall TConvertThread::UpdateCaption()
//      {
//        Form1->Caption = "Updated in a thread";
//      }
//---------------------------------------------------------------------------

__fastcall TConvertThread::TConvertThread(bool CreateSuspended,
    TStringList *pFiles, AnsiString DestFolder, bool Single, ConvertInfo *ci)
    : TThread(CreateSuspended)
{
    this->pFiles = pFiles;
    this->DestFolder = DestFolder;
    this->Single = Single;
    this->ci = *ci;
    k = pFiles->Count;
    OpCode2 = mrYes;
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::Execute()
{
    WIN32_FIND_DATA wfd;
    HANDLE hFind;
    int ItemIndex = 0;

    Synchronize(ShowForm);

    for (; i < k; i++)
    {
        char DestFile[MAX_PATH];
        lstrcpy(DestFile, DestFolder.c_str());
        if (!Single)
        {
            if (DestFile[lstrlen(DestFile) - 1] != '\\')
                lstrcat(DestFile, "\\");
            lstrcat(DestFile, ExtractFileName(pFiles->Strings[i]).c_str());
            char *dot = strchr(DestFile, '.');
            if (dot) *dot = 0;
            if (ci.ToVox)
                lstrcat(DestFile, ".vox");
            else
                lstrcat(DestFile, ".wav");
        }

        if ((hFind = FindFirstFile(DestFile, &wfd)) != INVALID_HANDLE_VALUE)
        {
            FindClose(hFind);
            Synchronize(OverwritePrompt);
            switch (OpCode2)
            {
                case mrYes: break;
                case mrNo:
                    ItemIndex++;
                    Synchronize(SetProgress);
                    continue;
                case mrCancel:
                    Synchronize(HideForm);
                    return;
            }
        }

        if (!ConvertFile(pFiles->Strings[i].c_str(), DestFile, &ci))
        {
            Synchronize(AbortRetryIgnore);
            switch (OpCode)
            {
                case IDABORT:
                    DeleteFile(DestFile);
                    Synchronize(HideForm);
                    return;
                case IDRETRY:
                    i--;
                    continue;
            }
            ItemIndex++;
        }
        else
            MainForm->WAVList->Items->Delete(ItemIndex);
        Synchronize(SetProgress);
    }
    Synchronize(HideForm);
    Synchronize(ShowOK);
    delete pFiles;
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::ShowForm()
{
    ProgressForm->Thread = this;
    MainForm->Enabled = false;
    OverwriteDlg->ForAll->Checked = false;
    ProgressForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::HideForm()
{
    ProgressForm->Hide();
    MainForm->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::SetProgress()
{
    ProgressForm->ProgressBar->Position = i * 100 / k;
    ProgressForm->NumFiles->Caption =
        AnsiString(i) + "  " + k;
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::AbortRetryIgnore()
{
    OpCode = Application->MessageBox(("    \n" +
        pFiles->Strings[i]).c_str(), "", MB_ABORTRETRYIGNORE | MB_ICONSTOP);
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::OverwritePrompt()
{
    if (OverwriteDlg->ForAll->Checked) return;
    OverwriteDlg->FileName->Caption = pFiles->Strings[i];
    OverwriteDlg->ForAll->Checked = false;
    OpCode2 = OverwriteDlg->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TConvertThread::ShowOK()
{
    Application->MessageBox("    ",
        "ConvertWAV", MB_OK | MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------

