//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "Procs.h"
#include "ConvertThread.h"
#include "TPlayWavDlg.h"
#include "TConvertDlg.h"
#include "TAboutBox.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "shellctrls"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    IniFile = new TIniFile(
        ExtractFileDir(Application->ExeName) + "\\ConvertWav.ini");
    TreeView->AlphaSort(true);
    DragAcceptFiles(Handle, TRUE);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
    if (Application->MessageBox("   ?",
            "ConvertWAV", MB_YESNO | MB_ICONQUESTION) == IDYES)
        CanClose = true;
    else CanClose = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iExitClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListAddFolder(TObject *Sender,
      TShellFolder *AFolder, bool &CanAdd)
{
	if (!AFolder->IsFolder())
    {
        AnsiString Ext = ExtractFileExt(AFolder->PathName()).LowerCase();
		CanAdd = (Ext == ".wav") || (Ext == ".vox");
    }
    else
    {
        AnsiString OldDir = GetCurrentDir();
        if (SetCurrentDir(AFolder->PathName()))
        {
            CanAdd = true;
            SetCurrentDir(OldDir);
        }
        else CanAdd = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
    Accept = (Source == FileList) || (Source == TreeView);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FindFiles(AnsiString FolderName)
{
    WIN32_FIND_DATA wfd;
    char FindPath[MAX_PATH];
    if (FolderName[FolderName.Length()] != '\\')
        FolderName += "\\";
    lstrcpy(FindPath, FolderName.c_str());
    lstrcat(FindPath, "*.*");
    HANDLE hFind = FindFirstFile(FindPath, &wfd);
    if (hFind == INVALID_HANDLE_VALUE) return;
    do
    {
        if (!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
            AddFile((FolderName + wfd.cFileName).c_str());
    } while (FindNextFile(hFind, &wfd));
    FindClose(hFind);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
    if (Source == TreeView)
    {
        FindFiles(TreeView->SelectedFolder()->PathName());
        return;
    }
    else if (Source != FileList) return;
    int i = 0, j, k = FileList->Items->Count;
    for (; i < k; i++)
    {
        if (!FileList->Items->Item[i]->Selected) continue;
        AnsiString FolderName = FileList->Folders[i]->PathName();
        char FileName[MAX_PATH];
        if (FileList->Folders[i]->IsFolder())
            FindFiles(FolderName);
        {
            lstrcpy(FileName, FolderName.c_str());
            AddFile(FileName);
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DelItemsBtnClick(TObject *Sender)
{
    int i = 0, k = WAVList->Items->Count;
    if (!k || Application->MessageBox(
        "    ?", "",
        MB_YESNO | MB_ICONWARNING) != IDYES) return;
    for (; i < k; i++)
    {
        if (!WAVList->Items->Item[i]->Selected) continue;
        WAVList->Items->Delete(i--);
        k--;
    }
    WAVList->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
    DelItemsBtn->Enabled = WAVList->SelCount;
    iDel->Enabled = WAVList->SelCount;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListInsert(TObject *Sender, TListItem *Item)
{
    SendMessage(WAVList->Handle, WM_VSCROLL, SB_BOTTOM, 0);
    UpdateFileCount(WAVList->Items->Count);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSelectAllClick(TObject *Sender)
{
    if (ActiveControl == FileList)
    {
        int i = 0, k = FileList->Items->Count;
        for (; i < k; i++)
            FileList->Items->Item[i]->Selected = true;
    }
    else if (ActiveControl == WAVList)
    {
        int i = 0, k = WAVList->Items->Count;
        for (; i < k; i++)
            WAVList->Items->Item[i]->Selected = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iDelClick(TObject *Sender)
{
    DelItemsBtnClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iCopyClick(TObject *Sender)
{
    WAVListDragDrop(NULL, FileList, 0, 0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CopyBtnClick(TObject *Sender)
{
    WAVListDragDrop(NULL, FileList, 0, 0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListClick(TObject *Sender)
{
    iCopy->Enabled = FileList->SelCount;
    CopyBtn->Enabled = FileList->SelCount;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if (Key == VK_RETURN) FileListDblClick(NULL);
    FileListClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListKeyUp(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    FileListClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    FileListClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iLargeIconsClick(TObject *Sender)
{
    iLargeIcons->Checked = true;
    iSmallIcons->Checked = false;
    iList->Checked = false;
    FileList->ViewStyle = vsIcon;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSmallIconsClick(TObject *Sender)
{
    iLargeIcons->Checked = false;
    iSmallIcons->Checked = true;
    iList->Checked = false;
    FileList->ViewStyle = vsSmallIcon;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iListClick(TObject *Sender)
{
    iLargeIcons->Checked = false;
    iSmallIcons->Checked = false;
    iList->Checked = true;
    FileList->ViewStyle = vsList;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListDblClick(TObject *Sender)
{
    if (!FileList->Selected) return;
    AnsiString PathName = FileList->SelectedFolder()->PathName();
    if (ExtractFileExt(PathName).LowerCase() == ".wav" ||
        ExtractFileExt(PathName).LowerCase() == ".vox")
        PlayWavDlg->Play(PathName);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListDblClick(TObject *Sender)
{
    if (WAVList->Selected)
        PlayWavDlg->Play(WAVList->Selected->Caption);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iConvertClick(TObject *Sender)
{
    ConvertInfo ci;
    if (WAVList->Items->Count == 1)
    {
        ConvertDlg->SrcFile->Text = WAVList->Items->Item[0]->Caption;
        ConvertDlg->DestFile->Text = ConvertDlg->SrcFile->Text;
        ConvertDlg->RadioButton2->Checked = true;
        ConvertDlg->FormatsClick(NULL);
    }
    else
    {
        ConvertDlg->SrcFile->Text = "";
        ConvertDlg->DestFile->Text = "";
        ConvertDlg->RadioButton1->Checked = true;
    }
    if (ConvertDlg->ShowModal() != mrOk) return;
    ci.Fmt = ConvertDlg->Format;
    ci.Normalize = ConvertDlg->Normalize->Checked;
    ci.UseFilter = ConvertDlg->UseFilter->Checked;
    ci.ToVox = ConvertDlg->Formats->ItemIndex;
    ci.VoxFreq = ConvertDlg->VoxFreq;
    if (ConvertDlg->RadioButton1->Checked)
    {
        if (!WAVList->Items->Count)
        {
            Application->MessageBox("       ",
                "  ", MB_OK | MB_ICONEXCLAMATION);
            return;
        }
        int i = 0, k = WAVList->Items->Count;
        TStringList *pFiles = new TStringList();
        for (; i < k; i++)
            pFiles->Add(WAVList->Items->Item[i]->Caption);
        new TConvertThread(false, pFiles, ConvertDlg->SaveFolder->Text, false, &ci);
    }
    else
    {
        TStringList *pFiles = new TStringList();
        pFiles->Add(ConvertDlg->SrcFile->Text);
        new TConvertThread(false, pFiles, ConvertDlg->DestFile->Text, true, &ci);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListDeletion(TObject *Sender,
      TListItem *Item)
{
    UpdateFileCount(WAVList->Items->Count - 1);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iClearListClick(TObject *Sender)
{
    WAVList->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMDropFiles(TMessage Message)
{
    int i = 0, j, k = DragQueryFile((void *)Message.WParam, -1, NULL, 0);
    char FileName[MAX_PATH];
    for (; i < k; i++)
    {
        DragQueryFile((void *)Message.WParam, i, FileName, MAX_PATH);
        AddFile(FileName);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddFile(LPTSTR FileName)
{
    int i = 0, k = WAVList->Items->Count;
    AnsiString Ext = ExtractFileExt(FileName).LowerCase();
    if (Ext == ".wav" || Ext == ".vox")
    {
        for (i = 0; i < k; i++)
            if (WAVList->Items->Item[i]->Caption == FileName)
                break;
        if (i < k) return;
        TListItem *li = WAVList->Items->Add();
        li->Caption = FileName;
        WIN32_FIND_DATA wfd;
        ZeroMemory(&wfd, sizeof(wfd));
        FindClose(FindFirstFile(FileName, &wfd));
        li->SubItems->Add(SizeToStr(wfd.nFileSizeLow));
        li->SubItems->Add(FormatToStr(FileName));
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateFileCount(int Count)
{
    StatusBar->Panels->Items[1]->Text = AnsiString(" : ") + Count;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ConvertBtnClick(TObject *Sender)
{
    iConvertClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayBtnClick(TObject *Sender)
{
    if (ActiveControl == FileList)
        FileListDblClick(NULL);
    else if (ActiveControl == WAVList)
        WAVListDblClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iPlayClick(TObject *Sender)
{
    PlayBtnClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iAboutClick(TObject *Sender)
{
    AboutBox->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WAVListKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if (Key == VK_RETURN) WAVListDblClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BackBtnClick(TObject *Sender)
{
    FileList->Back();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefreshBtnClick(TObject *Sender)
{
    FileList->Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iRefreshClick(TObject *Sender)
{
    FileList->Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    IniFile->WriteString("Parameters", "LastSelectedPath", PathBuf);
    IniFile->WriteString("Parameters", "SaveFolder", ConvertDlg->SaveFolder->Text);
    IniFile->WriteString("Parameters", "Path", TreeView->Path);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
    AnsiString t = IniFile->ReadString("Parameters", "LastSelectedPath", "");
    ConvertDlg->SaveFolder->Text = IniFile->ReadString("Parameters", "SaveFolder", "");
    lstrcpy(PathBuf, t.c_str());
    t = IniFile->ReadString("Parameters", "Path", "");
    if (!t.IsEmpty()) TreeView->Path = t;
}
//---------------------------------------------------------------------------

