//---------------------------------------------------------------------------


#pragma hdrstop

#include "MyConvertWAV.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------
inline int min(int a, int b)
{
    if (a < b) return a;
    return b;
}
//---------------------------------------------------------------------------
void SamplesToFloat(float **Data, LPVOID Samples, LPWAVEFORMATEX DataFmt, int N)
{
    int k = DataFmt->nChannels, i, n = 0;
    BYTE *pbInput;
    short *pwInput;
	float *p;

    for (; n < k; n++)
    {
        pbInput = ((BYTE *)Samples) + n;
        pwInput = ((short *)Samples) + n;
		p = Data[n];
        if (DataFmt->wBitsPerSample == 8)
            for (i = 0; i < N; i++, pbInput += k)
                *p++ = (float)(-128 + *pbInput) / 128;
        else
            for (i = 0; i < N; i++, pwInput += k)
                *p++ = (float)*pwInput / 32768;
    }
}
//---------------------------------------------------------------------------
void FloatToSamples(float **Data, LPVOID Samples, LPWAVEFORMATEX DataFmt, int N)
{
    int k = DataFmt->nChannels, i, n = 0;
    BYTE *pbOutput;
    short *pwOutput;
	float *p;

    for (; n < k; n++)
    {
        pbOutput = ((BYTE *)Samples) + n;
        pwOutput = ((short *)Samples) + n;
		p = Data[n];
        if (DataFmt->wBitsPerSample == 8)
            for (i = 0; i < N; i++, pbOutput += k)
            {
                if (*p > 1.0f) *p = 1.0f;
                if (*p < -1.0f) *p = -1.0f;
                *pbOutput = (BYTE)(*p++ * 127 + 128);
            }
        else
            for (i = 0; i < N; i++, pwOutput += k)
            {
                if (*p > 1.0f) *p = 1.0f;
                if (*p < -1.0f) *p = -1.0f;
                *pwOutput = (short)(*p++ * 32767);
            }
    }
}
//---------------------------------------------------------------------------
BOOL MyConvertWAV(TCHAR *SrcFile, TCHAR *DestFile, LPWAVEFORMATEX DestFmt,
				BOOL UseFilter, BOOL Normalize)
{
	BOOL Res = TRUE;
	WAVEFORMATEX SrcFmt;
	HANDLE hSrcFile = 0, hDestFile = 0;
	DWORD FileSize, DataSize;
	DWORD Read;
	WAVHeader whdr;

	try
	{
		if ((hSrcFile = CreateFile(SrcFile, GENERIC_READ, FILE_SHARE_READ,
			NULL, OPEN_EXISTING, 0, 0)) == INVALID_HANDLE_VALUE)
			throw 0;
		FileSize = GetFileSize(hSrcFile, NULL);
        if (!GetFileFormat(hSrcFile, &whdr, &SrcFmt, DataSize)) throw 0;
		if ((hDestFile = CreateFile(DestFile, GENERIC_WRITE, 0, NULL,
			CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0)) == INVALID_HANDLE_VALUE)
			throw 0;
		WriteFile(hDestFile, &whdr, sizeof(whdr), &Read, NULL);
		WriteFile(hDestFile, DestFmt, whdr.fmt.Size, &Read, NULL);
		if (!CopyFileBlock(hSrcFile, hDestFile, sizeof(whdr) + whdr.fmt.Size,
			SetFilePointer(hSrcFile, 0, NULL, FILE_CURRENT) - sizeof(WAVSection)))
			throw 0;
		if (!ConvertData(hSrcFile, hDestFile, &SrcFmt, DestFmt, DataSize,
            UseFilter, Normalize))
			throw 0;
		if (!CopyFileBlock(hSrcFile, hDestFile,
			SetFilePointer(hSrcFile, 0, NULL, FILE_CURRENT),
			FileSize)) throw 0;

		FileSize = GetFileSize(hDestFile, NULL);
		FileSize -= 8;
		SetFilePointer(hDestFile, 4, NULL, FILE_BEGIN);
		WriteFile(hDestFile, &FileSize, 4, &Read, NULL);
	}
	catch (int)
	{
		Res = FALSE;
	}

	CloseHandle(hDestFile);
	CloseHandle(hSrcFile);

	return Res;
}
//---------------------------------------------------------------------------
BOOL ConvertData(HANDLE hSrcFile, HANDLE hDestFile,
				 LPWAVEFORMATEX SrcFmt, LPWAVEFORMATEX DestFmt,
                 DWORD DataSize, BOOL UseFilter, BOOL Normalize)
{
	int nSrcSamples = DataSize / SrcFmt->nBlockAlign;
	int Blocks = nSrcSamples / BLOCK_SAMPLES;
	int BlockRem = nSrcSamples % BLOCK_SAMPLES;
	int DestBlockSamples =
		int(__int64(BLOCK_SAMPLES) * DestFmt->nSamplesPerSec / SrcFmt->nSamplesPerSec);
	int DestBlockRem = DestBlockSamples * BlockRem / BLOCK_SAMPLES;
	int DestDataSize = (DestBlockSamples * Blocks + DestBlockRem) *
		DestFmt->nBlockAlign;
	int i, j, N;
	float **fSrcBuf = new float *[SrcFmt->nChannels];
	float **fDestBuf = new float *[DestFmt->nChannels];
	float *re_c = new float[FFT_SIZE];
	float *im_c = new float[FFT_SIZE];
	float **fbuf;
	int FilterChannels;
	int SrcBufSize = BLOCK_SAMPLES * SrcFmt->nBlockAlign;
	int DestBufSize = DestBlockSamples * DestFmt->nBlockAlign;
	LPBYTE pSrcBuf = new BYTE[SrcBufSize];
	LPBYTE pDestBuf = new BYTE[DestBufSize];
	DWORD Read;
	BOOL Res = TRUE;
	int Max = 0;
    WAVSection wsec;

    memcpy(wsec.Name, "data", 4);
    wsec.Size = DestDataSize;
    WriteFile(hDestFile, &wsec, sizeof(wsec), &Read, NULL);

	if (Normalize)
        AnalyzeData(hSrcFile, DataSize, SrcFmt->wBitsPerSample == 16, Max);

	if (BlockRem) Blocks++;

	for (i = 0; i < SrcFmt->nChannels; i++)
		fSrcBuf[i] = new float[BLOCK_SAMPLES];
	for (i = 0; i < DestFmt->nChannels; i++)
		fDestBuf[i] = new float[DestBlockSamples];

	if (SrcFmt->nSamplesPerSec > DestFmt->nSamplesPerSec)
	{
		FilterChannels = SrcFmt->nChannels;
		InitFilter(re_c, im_c, FIR_KERNEL_SIZE,
			0.5f * DestFmt->nSamplesPerSec / SrcFmt->nSamplesPerSec);
	}
	else
	{
		FilterChannels = DestFmt->nChannels;
		InitFilter(re_c, im_c, FIR_KERNEL_SIZE,
			0.5f * SrcFmt->nSamplesPerSec / DestFmt->nSamplesPerSec);
	}

	fbuf = new float *[FilterChannels];
	for (i = 0; i < FilterChannels; i++)
	{
		fbuf[i] = new float[FIR_KERNEL_SIZE - 1];
		ZeroMemory(fbuf[i], sizeof(float) * (FIR_KERNEL_SIZE - 1));
	}

	for (i = 0; i < Blocks; i++)
	{
		N = min(SrcBufSize, (int)DataSize - i * SrcBufSize);
		ReadFile(hSrcFile, pSrcBuf, N, &Read, NULL);
		if (Read != N)
		{
			Res = FALSE;
			break;
		}

        if (Normalize)
		    NormalizeData(pSrcBuf, SrcBufSize, SrcFmt, Max);

		if (!BlockRem || i < Blocks - 1)
			SamplesToFloat(fSrcBuf, pSrcBuf, SrcFmt, BLOCK_SAMPLES);
		else
		{
			for (j = 0; j < SrcFmt->nChannels; j++)
				ZeroMemory(fSrcBuf[j], BLOCK_SAMPLES * sizeof(float));
			SamplesToFloat(fSrcBuf, pSrcBuf, SrcFmt, BlockRem);
		}

		if (SrcFmt->nSamplesPerSec > DestFmt->nSamplesPerSec)
		{
            if (UseFilter)
    			Filter(re_c, im_c, fSrcBuf, fbuf, FilterChannels, BLOCK_SAMPLES);
			Resample(fSrcBuf, SrcFmt, fDestBuf, DestFmt, BLOCK_SAMPLES,
				DestBlockSamples);
		}
		else
		{
			Resample(fSrcBuf, SrcFmt, fDestBuf, DestFmt, BLOCK_SAMPLES,
				DestBlockSamples);
            if (UseFilter)
    			Filter(re_c, im_c, fDestBuf, fbuf, FilterChannels, DestBlockSamples);
		}

		if (!BlockRem || i < Blocks - 1)
			FloatToSamples(fDestBuf, pDestBuf, DestFmt, DestBlockSamples);
		else
			FloatToSamples(fDestBuf, pDestBuf, DestFmt, DestBlockRem);

		N = min(DestBufSize, DestDataSize - i * DestBufSize);
		WriteFile(hDestFile, pDestBuf, N, &Read, NULL);
		if (Read != N)
		{
			Res = FALSE;
			break;
		}
	}

	for (i = 0; i < FilterChannels; i++)
		delete fbuf[i];
	delete fbuf;
	delete im_c;
	delete re_c;
	for (i = 0; i < DestFmt->nChannels; i++)
		delete fDestBuf[i];
	for (i = 0; i < SrcFmt->nChannels; i++)
		delete fSrcBuf[i];
	delete fDestBuf;
	delete fSrcBuf;
	delete pDestBuf;
	delete pSrcBuf;

	return Res;
}
//---------------------------------------------------------------------------
BOOL CopyFileBlock(HANDLE hSrcFile, HANDLE hDestFile, DWORD BegOffs, DWORD EndOffs)
{
	DWORD CopySize = EndOffs - BegOffs;
	int Blocks = CopySize / COPY_BLOCK_SIZE;
	int BlockRem = CopySize % COPY_BLOCK_SIZE;
	LPBYTE Buf = new BYTE[COPY_BLOCK_SIZE];
	DWORD Read;
	int i, N;

	if (!CopySize) return TRUE;
	if (BlockRem) Blocks++;
	DWORD OldPtr = SetFilePointer(hSrcFile, 0, NULL, FILE_CURRENT);
    SetFilePointer(hSrcFile, BegOffs, NULL, FILE_BEGIN);

	for (i = 0; i < Blocks; i++)
	{
		N = min(COPY_BLOCK_SIZE, CopySize - i * COPY_BLOCK_SIZE);
		ReadFile(hSrcFile, Buf, N, &Read, NULL);
		if (Read != N) return FALSE;
		WriteFile(hDestFile, Buf, N, &Read, NULL);
		if (Read != N) return FALSE;
	}

	SetFilePointer(hSrcFile, OldPtr, 0, FILE_BEGIN);

	delete Buf;
	return TRUE;
}
//---------------------------------------------------------------------------
BOOL AnalyzeData(HANDLE hFile, DWORD DataSize, bool Is16Bit, int &Max)
{
	int Blocks = DataSize / COPY_BLOCK_SIZE, t = 0;
	int BlockRem = DataSize % COPY_BLOCK_SIZE;
	LPBYTE Buf = new BYTE[COPY_BLOCK_SIZE];
	DWORD Read;
	int i, N, M = Is16Bit ? COPY_BLOCK_SIZE >> 1 : COPY_BLOCK_SIZE;

	if (BlockRem) Blocks++;
	DWORD OldPtr = SetFilePointer(hFile, 0, NULL, FILE_CURRENT);

	for (i = 0; i < Blocks; i++)
	{
		N = min(COPY_BLOCK_SIZE, DataSize - i * COPY_BLOCK_SIZE);
		ReadFile(hFile, Buf, N, &Read, NULL);
		if (Read != N) return FALSE;
		if (!Is16Bit)
		{
			__asm
			{
				mov		edx, t
				mov		esi, Buf
				mov		ecx, M
				cld
m1:				lodsb
				movzx	eax, al
				sub		eax, 128
				cmp		eax, edx
				jng		m2
				mov		edx, eax
				jmp		short m3
m2:				neg		eax
				cmp		eax, edx
				jng		m3
				mov		edx, eax
m3:				loop	m1
				mov		t, edx
			}
		}
		else
		{
			__asm
			{
				mov		edx, t
				mov		esi, Buf
				mov		ecx, M
				cld
m6:				lodsw
				movsx	eax, ax
				cmp		eax, edx
				jng		m7
				mov		edx, eax
				jmp		short m8
m7:				neg		eax
				cmp		eax, edx
				jng		m8
				mov		edx, eax
m8:				loop	m6
				mov		t, edx
			}
		}
	}

	Max = t;
	SetFilePointer(hFile, OldPtr, NULL, FILE_BEGIN);

	delete Buf;
	return TRUE;
}
//---------------------------------------------------------------------------
void NormalizeData(LPBYTE Data, DWORD DataSize, LPWAVEFORMATEX DataFmt, int Max)
{
	int i = 0, Channels = DataFmt->nChannels;
	int N = DataSize / DataFmt->nBlockAlign;
	for (; i < Channels; i++)
	{
		if (DataFmt->wBitsPerSample == 8)
		{
			__asm
			{
				mov		esi, Data
				add		esi, i
				mov		ebx, Channels
				dec		ebx
				cld
				mov		edx, Max
				or		edx, edx
				jz		m4
				mov		edi, esi
				mov		ecx, N
				push	ebp
				mov		ebp, ebx
				mov		ebx, edx
m5:				lodsb
				movzx	eax, al
				sub		eax, 128
				cdq
				imul	eax, 127
				idiv	ebx
				add		eax, 128
				stosb
				add		esi, ebp
				add		edi, ebp
				loop	m5
				pop		ebp
m4:
			}
		}
		else
		{
			__asm
			{
				mov		esi, i
				shl		esi, 1
				add		esi, Data
				mov		ebx, Channels
				dec		ebx
				shl		ebx, 1
				cld
				mov		edx, Max
				or		edx, edx
				jz		m10
				mov		edi, esi
				mov		ecx, N
				push	ebp
				mov		ebp, ebx
				mov		ebx, edx
m9:				lodsw
				movsx	eax, ax
				cdq
				imul	eax, 32767
				idiv	ebx
				stosw
				add		esi, ebp
				add		edi, ebp
				loop	m9
				pop		ebp
m10:
			}
		}
	}
}
//---------------------------------------------------------------------------
void ResampleChannel(float *q, float *p, int N, int M)
{
	int i, j, k = N / M;
	double r = fmod((float)N / M, 1), t = 0.0;
	float a, b;

	if (k)
	{
		*p++ = *q;
		q += k;
		t += r;
		i = 1;
		j = k;
	}
	else
	{
		a = 0.5f * (*(q + 1) - *q);
		for (i = 0; !t; i++)
		{
			*p++ = a * float(t * t + t) + *q;
			t += r;
		}
		t -= 1.0;
		q++;
		j = 1;
	}
	for (; i < M && j < N - 1; i++)
	{
		a = 0.5f * (*(q + 1) + *(q - 1)) - *q;
		b = 0.5f * (*(q + 1) - *(q - 1));
		*p++ = float(a * t * t + b * t + *q);
		q += k;
		j += k;
		t += r;
		if (t >= 1.0)
		{
			t -= 1.0;
			q++;
			j++;
		}
	}
	for (; i < M; i++)
	{
		a = 0.5f * (*(q - 1) - *q);
		b = 0.5f * (*q - *(q - 1));
		*p++ = float(a * t * t + b * t + *q);
		q += k;
		t += r;
		if (t >= 1.0)
		{
			t -= 1.0;
			q++;
		}
	}
}
//---------------------------------------------------------------------------
void Resample(float **SrcBuf, LPWAVEFORMATEX SrcFmt, float **DestBuf,
			  LPWAVEFORMATEX DestFmt, int N, int M)
{
	int n = SrcFmt->nChannels, m = DestFmt->nChannels;

	if (m == 1 && n == 1)
		ResampleChannel(SrcBuf[0], DestBuf[0], N, M);
	else if (m == 2 && n == 1)
	{
		ResampleChannel(SrcBuf[0], DestBuf[0], N, M);
		memcpy(DestBuf[1], DestBuf[0], sizeof(float) * M);
	}
	else if (m == 1 && n == 2)
	{
		float *buf = new float[M];
		float *p = DestBuf[0], *q = buf;
		ResampleChannel(SrcBuf[0], p, N, M);
		ResampleChannel(SrcBuf[1], q, N, M);
		for (int i = 0; i < M; i++)
			*p++ = 0.5f * (*p + *q++);
		delete buf;
	}
	else if (m == 2 && n == 2)
	{
		ResampleChannel(SrcBuf[0], DestBuf[0], N, M);
		ResampleChannel(SrcBuf[1], DestBuf[1], N, M);
	}
}
//---------------------------------------------------------------------------
void InitFilter(float *re_c, float *im_c, int M, float fc)
{
	fc *= 0.9f;
	float K = 0.0f, c = float(2 * M_PI * fc);
	int m2 = M >> 1, i;

	for (i = 0; i < M; i++)
	{
		if (!(i - m2))
			re_c[i] = c;
		else
			re_c[i] = float(sin(c * (i - m2)) / (i - m2));
		re_c[i] *= float(0.54 - 0.46 * cos(2 * M_PI * i / M));
		K += re_c[i];
	}

	for (i = 0; i < M; i++)
		re_c[i] /= K;

	ZeroMemory(re_c + M, sizeof(float) * (FFT_SIZE - M));
	ForwardFFT(FFT_SIZE, re_c, im_c);
}
//---------------------------------------------------------------------------
void Filter(float *re_c, float *im_c, float **DataBuf, float **FltBuf,
			int Channels, int nSamples)
{
	int Blocks = nSamples / FIR_BLOCK_SIZE;
	int BlockRem = nSamples % FIR_BLOCK_SIZE;
	float *re_x = new float[FFT_SIZE];
	float *im_x = new float[FFT_SIZE];
	float tr, ti;
	int i, j, k, n;
	float *p;

	if (BlockRem) Blocks++;

	for (i = 0; i < Channels; i++)
	{
		p = DataBuf[i];
		for (k = 0; k < Blocks; k++)
		{
			n = min(FIR_BLOCK_SIZE, nSamples - k * FIR_BLOCK_SIZE);
			ZeroMemory(re_x + n, sizeof(float) * (FFT_SIZE - n));
			memcpy(re_x, p, sizeof(float) * n);

			ForwardFFT(FFT_SIZE, re_x, im_x);

			for (j = 0; j < FFT_SIZE; j++)
		    {
	            tr = re_x[j];
		        ti = im_x[j];
			    re_x[j] = re_c[j] * tr - im_c[j] * ti;
	            im_x[j] = re_c[j] * ti + im_c[j] * tr;
		    }

			InverseFFT(FFT_SIZE, re_x, im_x);

			memcpy(p, re_x, n * sizeof(float));
			for (j = 0; j < n; j++)
		        *p++ = re_x[j] + FltBuf[i][j];

			if (n >= FIR_KERNEL_SIZE - 1)
				memcpy(FltBuf[i], re_x + n, sizeof(float) * (FIR_KERNEL_SIZE - 1));
			else
			{
				memmove(FltBuf[i], FltBuf[i] + n, sizeof(float) * (FIR_KERNEL_SIZE - 1 - n));
				for (j = 0; j < FIR_KERNEL_SIZE - 1 - n; j++)
					FltBuf[i][j] += re_x[j + n];
				memcpy(FltBuf[i] + FIR_KERNEL_SIZE - 1 - n,
					re_x + FIR_BLOCK_SIZE, sizeof(float) * n);
			}
		}
	}

	delete im_x;
	delete re_x;
}
//---------------------------------------------------------------------------
void ForwardFFT(int N, float *re_x, float *im_x)
{
    int N2 = N >> 1, N4 = N >> 2;
    int i = 0, im, ip2, ipm;

    for (; i < N2; i++)
    {
        re_x[i] = re_x[i << 1];
        im_x[i] = re_x[(i << 1) + 1];
    }

    FFT(N2, re_x, im_x);

    for (i = 1; i < N4; i++)
    {
        im = N2 - i;
        ip2 = N2 + i;
        ipm = N2 + im;
        re_x[ip2] = 0.5f * (im_x[i] + im_x[im]);
        re_x[ipm] = re_x[ip2];
        im_x[ip2] = -0.5f * (re_x[i] - re_x[im]);
        im_x[ipm] = -im_x[ip2];
        re_x[i] = 0.5f * (re_x[i] + re_x[im]);
        re_x[im] = re_x[i];
        im_x[i] = 0.5f * (im_x[i] - im_x[im]);
        im_x[im] = -im_x[i];
    }
    re_x[N4 * 3] = im_x[N4];
    re_x[N2] = im_x[0];
    im_x[N4 * 3] = 0;
    im_x[N2] = 0;
    im_x[N4] = 0;
    im_x[0] = 0;

    int l = (int)ceil(log(N) / log(2));
    int le = 1 << l;
    int le2 = le >> 1;
    float ur = 1, ui = 0;
    float sr = (float)cos(M_PI / le2);
    float si = (float)-sin(M_PI / le2);
    float tr, ti;

    for (int j = 1; j <= le2; j++)
    {
        for (i = j - 1; i < N; i += le)
        {
            ip2 = i + le2;
            tr = re_x[ip2] * ur - im_x[ip2] * ui;
            ti = re_x[ip2] * ui + im_x[ip2] * ur;
            re_x[ip2] = re_x[i] - tr;
            im_x[ip2] = im_x[i] - ti;
            re_x[i] = re_x[i] + tr;
            im_x[i] = im_x[i] + ti;
        }
        tr = ur;
        ur = tr * sr - ui * si;
        ui = tr * si + ui * sr;
    }
}
//---------------------------------------------------------------------------
void InverseFFT(int N, float *re, float *im)
{
    int i = (N >> 1) + 1, k = N - i;
    float n1 = 1.0f / N;

    for (; i < N; i++, k--)
    {
        re[i] = re[k];
        im[i] = -im[k];
    }

    for (i = 0; i < N; i++)
        re[i] += im[i];

    ForwardFFT(N, re, im);

    for (i = 0; i < N; i++)
        re[i] = (re[i] + im[i]) * n1;
}
//---------------------------------------------------------------------------
void FFT(int N, float *re, float *im)
{
	float tr, ti, ur, ui, sr, si;
	int nm1 = N - 1, nd2 = N >> 1, m = log2(N);
	int i, j = nd2, k, l, le, le2, jm1, ip;

	for (i = 1; i < nm1; i++)
	{
		if (i < j)
		{
			tr = re[j];
			ti = im[j];
			re[j] = re[i];
			im[j] = im[i];
			re[i] = tr;
			im[i] = ti;
		}
		k = nd2;
		while (k <= j)
		{
			j -= k;
			k >>= 1;
		}
		j += k;
	}

	for (l = 1; l <= m; l++)
	{
		le = 1 << l;
		le2 = le >> 1;
		ur = 1.0;
		ui = 0.0;
		sr = (float)cos(M_PI / le2);
		si = (float)-sin(M_PI / le2);
		for (j = 1; j <= le2; j++)
		{
			jm1 = j - 1;
			for (i = jm1; i <= nm1; i += le)
			{
				ip = i + le2;
				tr = re[ip] * ur - im[ip] * ui;
				ti = re[ip] * ui + im[ip] * ur;
				re[ip] = re[i] - tr;
				im[ip] = im[i] - ti;
				re[i] += tr;
				im[i] += ti;
			}
			tr = ur;
			ur = tr * sr - ui * si;
			ui = tr * si + ui * sr;
		}
	}
}
//---------------------------------------------------------------------------
int log2(int x)
{
    __asm bsr eax, x
}
//---------------------------------------------------------------------------
bool GetFileFormat(HANDLE hSrcFile, WAVHeader *whdr, LPWAVEFORMATEX wfx, DWORD &DataSize)
{
	DWORD Read, FileSize = GetFileSize(hSrcFile, NULL);
	WAVSection wsec;

    ReadFile(hSrcFile, whdr, sizeof(WAVHeader), &Read, NULL);
    if (memcmp(whdr->RIFF.Name, "RIFF", 4)) return false;
    if (FileSize - whdr->RIFF.Size != 8) return false;
    if (memcmp(whdr->WAVE, "WAVE", 4)) return false;
    if (memcmp(whdr->fmt.Name, "fmt ", 4)) return false;
    ReadFile(hSrcFile, wfx, 0x10, &Read, NULL);
    SetFilePointer(hSrcFile, whdr->fmt.Size - 0x10, NULL, FILE_CURRENT);
    if (wfx->wFormatTag != WAVE_FORMAT_PCM) return false;
    if (wfx->nChannels != 1 && wfx->nChannels != 2 || (
        wfx->wBitsPerSample != 8 &&
        wfx->wBitsPerSample != 16)) return false;

    while (1)
    {
        ReadFile(hSrcFile, &wsec, sizeof(wsec), &Read, NULL);
        if (Read != sizeof(WAVSection)) return false;
        if (!memcmp(wsec.Name, "data", 4)) break;
        SetFilePointer(hSrcFile, wsec.Size, NULL, FILE_CURRENT);
    }

    DataSize = wsec.Size;
    wfx->cbSize = 0;

    return true;
}
//---------------------------------------------------------------------------

