//---------------------------------------------------------------------------


#pragma hdrstop

#include "Vox.h"
#include "MyConvertWAV.h"

#define BLOCK_SIZE  0x4000

//---------------------------------------------------------------------------

int SSTable[49] = {
16,17,19,21,23,25,28,31,34,37,41,45,50,55,
60,66,73,80,88,97,107,118,130,143,157,173,190,
209,230,253,279,307,337,371,408,449,494,544,
598,658,724,796,876,963,1060,1166,1282,1411,1552};
int MLN[16] = {-1,-1,-1,-1,+2,+4,+6,+8,-1,-1,-1,-1,+2,+4,+6,+8};
int DTable1[49][16];
int DTable2[49][16];

#pragma package(smart_init)

void InitVoxTables()
{
    int i = 0, j, t;
    for (; i < 49; i++)
        for (j = 0; j < 16; j++)
        {
            t = SSTable[i];
            t =
                t * ((j >> 2) & 1) +
                ((t * ((j >> 1) & 1)) >> 1) +
                ((t * (j & 1)) >> 2) +
                (t >> 3);
            if ((j >> 3) & 1) t = -t;
            DTable1[i][j] = t;
            t = SSTable[i];
            t =
                (t * ((j & 4) >> 2)) +
                ((t >> 1) * ((j & 2) >> 1)) +
                ((t >> 2) * (j & 1)) +
                (t >> 3);
            if (j & 8) t = -t;
            DTable2[i][j] = t;
        }
}

inline char ADPCMsample(short Smp, int& ssp, int& x)
{
    int ss = SSTable[ssp], ln;
    int dn = (Smp >> 4) - x;
    if (dn < 0)
    {
        ln = 8;
        dn = -dn;
    } else ln = 0;
    if (dn >= ss)
    {
        ln |= 4;
        dn -= ss;
    }
    ss >>= 1;
    if (dn >= ss)
    {
        ln |= 2;
        dn -= ss;
    }
    ss >>= 1;
    if (dn >= ss) ln |= 1;

    x += DTable2[ssp][ln];

    ssp += MLN[ln];
    if (ssp < 0) ssp = 0;
    if (ssp > 48) ssp = 48;

    return ln;
}

BOOL VoxToPCM(LPTSTR pszSrcFile, LPTSTR pszDestFile, int Samplerate)
{
    int Xn, ssp;
    DWORD Read, Written, FileSize;
    WAVEFORMATEX wfx;

    HANDLE hSrcFile = CreateFile(pszSrcFile, GENERIC_READ, FILE_SHARE_READ,
        NULL, OPEN_EXISTING, 0, 0);
    if (hSrcFile == INVALID_HANDLE_VALUE) return false;
    HANDLE hDestFile = CreateFile(pszDestFile, GENERIC_WRITE, 0,
        NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
    if (hDestFile == INVALID_HANDLE_VALUE)
    {
        CloseHandle(hSrcFile);
        return false;
    }

    wfx.wFormatTag = WAVE_FORMAT_PCM;
    wfx.nChannels = 1;
    wfx.nSamplesPerSec = Samplerate;
    wfx.nAvgBytesPerSec = Samplerate << 1;
    wfx.nBlockAlign = 2;
    wfx.wBitsPerSample = 16;

    char *src_buf = new char[BLOCK_SIZE], *sp;
    short *dest_buf = new short[BLOCK_SIZE << 1], *dp;

    WriteFile(hDestFile, "RIFF", 4, &Written, NULL);
    WriteFile(hDestFile, "\0\0\0\0", 4, &Written, NULL);
    WriteFile(hDestFile, "WAVE", 4, &Written, NULL);
    WriteFile(hDestFile, "fmt ", 4, &Written, NULL);
    FileSize = 0x10;
    WriteFile(hDestFile, &FileSize, 4, &Written, NULL);
    WriteFile(hDestFile, &wfx, 0x10, &Written, NULL);
    WriteFile(hDestFile, "data", 4, &Written, NULL);
    WriteFile(hDestFile, "\0\0\0\0", 4, &Written, NULL);

    ssp = 0;
    Xn = 0;

    do
    {
        ReadFile(hSrcFile, src_buf, BLOCK_SIZE, &Read, NULL);
        __asm
        {
            pushad
            mov     ecx, Read
            mov     edx, Xn
            mov     esi, src_buf
            mov     edi, dest_buf
            push    ebp
            mov     ebp, ssp
            cld
m1:         lodsb
            movzx   eax, al
            push    eax
            shr     eax, 4
            mov     ebx, ebp
            shl     ebx, 6
            add     edx, [ebx+eax*4+DTable1]
            cmp     edx, -2048
            jnl     m2
            mov     edx, -2048
            jmp     short m3
m2:         cmp     edx, 2047
            jng     m3
            mov     edx, 2047
m3:         mov     ebx, eax
            mov     eax, edx
            shl     eax, 4
            stosw
            mov     eax, ebx
            add     ebp, [eax*4+MLN]
            jns     m4
            xor     ebp, ebp
m4:         cmp     ebp, 48
            jng     m5
            mov     ebp, 48
m5:         pop     eax
            and     eax, 0Fh
            mov     ebx, ebp
            shl     ebx, 6
            add     edx, [ebx+eax*4+DTable1]
            cmp     edx, -2048
            jnl     m6
            mov     edx, -2048
            jmp     short m7
m6:         cmp     edx, 2047
            jng     m7
            mov     edx, 2047
m7:         mov     ebx, eax
            mov     eax, edx
            shl     eax, 4
            stosw
            mov     eax, ebx
            add     ebp, [eax*4+MLN]
            jns     m8
            xor     ebp, ebp
m8:         cmp     ebp, 48
            jng     m9
            mov     ebp, 48
m9:         dec     ecx
            jcxz    m10
            jmp     m1
m10:        mov     eax, ebp
            pop     ebp
            mov     Xn, edx
            mov     ssp, eax
            popad
        }
        WriteFile(hDestFile, dest_buf, Read << 2, &Written, NULL);
    } while (Read == BLOCK_SIZE);

    FileSize = SetFilePointer(hDestFile, 0, NULL, FILE_CURRENT);
    FileSize -= 8;
    SetFilePointer(hDestFile, 4, NULL, FILE_BEGIN);
    WriteFile(hDestFile, &FileSize, 4, &Written, NULL);
    FileSize -= 0x24;
    SetFilePointer(hDestFile, 0x28, NULL, FILE_BEGIN);
    WriteFile(hDestFile, &FileSize, 4, &Written, NULL);

    delete dest_buf;
    delete src_buf;
    CloseHandle(hDestFile);
    CloseHandle(hSrcFile);

    return true;
}

BOOL PCMToVox(LPTSTR pszSrcFile, LPTSTR pszDestFile)
{
    WAVHeader whdr;
    WAVEFORMATEX wfx;
    DWORD DataSize;
    DWORD Read, Written, Count = 0, i;
    int ssp, x;

    HANDLE hSrcFile = CreateFile(pszSrcFile, GENERIC_READ, FILE_SHARE_READ,
        NULL, OPEN_EXISTING, 0, 0);
    if (hSrcFile == INVALID_HANDLE_VALUE) return false;

    if (!GetFileFormat(hSrcFile, &whdr, &wfx, DataSize))
    {
        CloseHandle(hSrcFile);
        return false;
    }

    HANDLE hDestFile = CreateFile(pszDestFile, GENERIC_WRITE, 0,
        NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
    if (hDestFile == INVALID_HANDLE_VALUE)
    {
        CloseHandle(hSrcFile);
        return false;
    }

    short *src_buf = new short[BLOCK_SIZE], *sp;
    char *dest_buf = new char[BLOCK_SIZE >> 2], *dp;

    x = 0;
    ssp = 0;

    do
    {
        ReadFile(hSrcFile, src_buf, BLOCK_SIZE, &Read, NULL);
        if (Count + Read > DataSize) Read = DataSize - Count;
        sp = src_buf;
        dp = dest_buf;
        for (i = 0; i < Read >> 2; i++)
            *dp++ = (
                ADPCMsample(*sp++, ssp, x) << 4) |
                ADPCMsample(*sp++, ssp, x);
        WriteFile(hDestFile, dest_buf, Read >> 2, &Written, NULL);
        Count += Read;
    } while (Read == BLOCK_SIZE && Count < DataSize);

    delete dest_buf;
    delete src_buf;

    CloseHandle(hDestFile);
    CloseHandle(hSrcFile);

    return true;
}

